// This header file is free for any kind of use commercial or non by any party.

// Author: Conor Stokes
// Purpose: Simple geometry primitive operations

#ifndef _HgeomH_
#define _HgeomH_

typedef Vector<3,1,1,1> vec3d; // 3d vec

// AABB primitive class
class CTriangle;
class CAABB {
	vec3d m_bounds[2]; // box bounds - min followed by max 
public:
	// default null constructor
//	CAABB() {}
	// box from min and max points
	CAABB(const vec3d& boxmin,const vec3d& boxmax) { m_bounds[0]=boxmin; m_bounds[1]=boxmax; }
	// calc aabb from trilist
	CAABB(const list<CTriangle>& triangles);	
	// returns a sub box of the octant specified, with the partition as the new corner. 
	float DistSqrd(const vec3d& point) const;
	// calculate the floating point error metric 
	float ErrorMetric() const {	return (vec3d(1.0f)+(m_bounds[1]-m_bounds[0])).Prod(); }
	// intersection scalar (used for weighting in building aabb)
	UINT32 LongestAxis() const;
	// mid-point
	vec3d MidPoint() const { return (m_bounds[0]+m_bounds[1])*.5f; }
	// copy 
	void Cpy(const CAABB& box) { m_bounds[0]=box.m_bounds[0]; m_bounds[1]=box.m_bounds[1];}
	// make this box encompass the current box as well as this one - returns * this
	void Encompass(const CAABB& encbox);
	// render this box
	void Render() const;
};


// triangle primitive class
class CTriangle {
	vec3d m_vnorms[3]; // vertex normals, for lighting
	vec3d m_verts[3]; // vertices 
	vec3d m_trinorm; // triangle normal
	float m_tri_d; // triangle's d plane component
public:
	// make from vert array
	CTriangle(const float* verts,const float* norms,const UINT16* indices);
	// get the aabb for this triangle (used for making group aabb)
	CAABB TriAABB() const;
	// distance squared to a point from the tri
	float DistSqrd(const vec3d& point) const;
	// calculate the midpoint
	vec3d MidPoint() const { return (m_verts[0]+m_verts[1]+m_verts[2])*(1.0f/3.0f); }
	// render this tri
	void Render() const;
};

// Sphere primitive class
class CSphere {
	vec3d m_origin; 
	float m_radsqrd;
public:
	// spere from vector and scalar
	CSphere(const vec3d& orgn,float rad) : m_origin(orgn),m_radsqrd(rad*rad) {}
	// does this intersect a tri?
	bool Intersects(const CTriangle& tri) const { return (tri.DistSqrd(m_origin)<=m_radsqrd); }
	// does this intersect an aabb?
	bool Intersects(const CAABB& aabb) const { return (aabb.DistSqrd(m_origin)<=m_radsqrd); }		
	// renders a sphere
	void Render() const;
};


#endif